@@ -12,6 +12,7 @@ SPDX-License-Identifier: GPL-2.0-or-later
 #include "wallpaper_integration.h"
 
 #include <config-kscreenlocker.h>
+#include <iostream>
 #include <kscreenlocker_greet_logging.h>
 
 #include <LayerShellQt/Window>
@@ -20,27 +21,22 @@ SPDX-License-Identifier: GPL-2.0-or-later
 #include <KAuthorized>
 #include <KConfigPropertyMap>
 #include <KCrash>
-#include <KDeclarative/KQuickAddons/QuickViewSharedEngine>
-#include <KDeclarative/QmlObjectSharedEngine>
 #include <KLocalizedContext>
 #include <KScreenDpms/Dpms>
 #include <KWindowSystem>
-#include <kdeclarative/kdeclarative.h>
+#include <PlasmaQuick/QuickViewSharedEngine>
 
 #include <KUser>
 // Plasma
 #include <KPackage/Package>
 #include <KPackage/PackageLoader>
-// KWayland
-#include <KWayland/Client/connection_thread.h>
-#include <KWayland/Client/event_queue.h>
-#include <KWayland/Client/registry.h>
 // Qt
 #include <QAbstractNativeEventFilter>
 #include <QClipboard>
 #include <QDBusConnection>
 #include <QKeyEvent>
 #include <QMimeData>
+#include <QSocketNotifier>
 #include <QThread>
 #include <QTimer>
 #include <qscreen.h>
@@ -52,11 +48,7 @@ SPDX-License-Identifier: GPL-2.0-or-later
 #include <QQuickItem>
 #include <QQuickView>
 
-#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
 #include <private/qtx11extras_p.h>
-#else
-#include <QX11Info>
-#endif
 // Wayland
 #include <wayland-client.h>
 #include <wayland-ksld-client-protocol.h>
@@ -68,6 +60,7 @@ SPDX-License-Identifier: GPL-2.0-or-later
 #include <xcb/xcb.h>
 
 #include "pamauthenticator.h"
+#include "pamauthenticators.h"
 
 // this is usable to fake a "screensaver" installation for testing
 // *must* be "0" for every public commit!
@@ -77,6 +70,8 @@ static const QString s_plasmaShellService = QStringLiteral("org.kde.plasmashell"
 static const QString s_osdServicePath = QStringLiteral("/org/kde/osdService");
 static const QString s_osdServiceInterface = QStringLiteral("org.kde.osdService");
 
+using namespace Qt::Literals;
+
 namespace ScreenLocker
 {
 // disable DrKonqi as the crash dialog blocks the restart of the locker
@@ -110,11 +105,7 @@ bool verifyPackageApi(const KPackage::Package &package)
 class FocusOutEventFilter : public QAbstractNativeEventFilter
 {
 public:
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-    bool nativeEventFilter(const QByteArray &eventType, void *message, long int *result) override
-#else
     bool nativeEventFilter(const QByteArray &eventType, void *message, qintptr *result) override
-#endif
     {
         Q_UNUSED(result)
         if (qstrcmp(eventType, "xcb_generic_event_t") != 0) {
@@ -128,6 +119,19 @@ public:
     }
 };
 
+class WallpaperItem : public WallpaperIntegration
+{
+    Q_OBJECT
+public:
+    explicit WallpaperItem(QQuickItem *parent = nullptr)
+        : WallpaperIntegration(parent)
+    {
+        setConfig(KScreenSaverSettingsBase::self()->sharedConfig());
+        setPluginName(KScreenSaverSettingsBase::self()->wallpaperPluginId());
+        init();
+    }
+};
+
 // App
 UnlockApp::UnlockApp(int &argc, char **argv)
     : QGuiApplication(argc, argv)
@@ -136,13 +140,16 @@ UnlockApp::UnlockApp(int &argc, char **argv)
     , m_testing(false)
     , m_ignoreRequests(false)
     , m_immediateLock(false)
-    , m_authenticator(new PamAuthenticator("kde", KUser().loginName(), this))
     , m_graceTime(0)
     , m_noLock(false)
     , m_defaultToSwitchUser(false)
-    , m_wallpaperIntegration(new WallpaperIntegration(this))
     , m_lnfIntegration(new LnFIntegration(this))
 {
+    auto interactive = std::make_unique<PamAuthenticator>(KSCREENLOCKER_PAM_SERVICE, KUser().loginName());
+    std::vector<std::unique_ptr<PamAuthenticator>> noninteractive;
+    noninteractive.push_back(std::make_unique<PamAuthenticator>(KSCREENLOCKER_PAM_FINGERPRINT_SERVICE, KUser().loginName(), PamAuthenticator::Fingerprint));
+    noninteractive.push_back(std::make_unique<PamAuthenticator>(KSCREENLOCKER_PAM_SMARTCARD_SERVICE, KUser().loginName(), PamAuthenticator::Smartcard));
+    m_authenticators = new PamAuthenticators(std::move(interactive), std::move(noninteractive), this);
     initialize();
 
     if (QX11Info::isPlatformX11()) {
@@ -154,7 +161,7 @@ UnlockApp::~UnlockApp()
 {
     // workaround QTBUG-55460
     // will be fixed when themes port to QQC2
-    for (auto view : qAsConst(m_views)) {
+    for (auto view : std::as_const(m_views)) {
         if (QQuickItem *focusItem = view->activeFocusItem()) {
             focusItem->setFocus(false);
         }
@@ -164,13 +171,8 @@ UnlockApp::~UnlockApp()
     if (m_ksldInterface) {
         org_kde_ksld_destroy(m_ksldInterface);
     }
-    if (m_ksldRegistry) {
-        delete m_ksldRegistry;
-    }
-    if (m_ksldConnection) {
-        m_ksldConnection->deleteLater();
-        m_ksldConnectionThread->quit();
-        m_ksldConnectionThread->wait();
+    if (m_display) {
+        wl_display_disconnect(m_display);
     }
 }
 
@@ -199,9 +201,13 @@ void UnlockApp::initialize()
 
     m_mainQmlPath = package.fileUrl("lockscreenmainscript");
 
-    m_wallpaperIntegration->setConfig(KScreenSaverSettingsBase::self()->sharedConfig());
-    m_wallpaperIntegration->setPluginName(KScreenSaverSettingsBase::self()->wallpaperPluginId());
-    m_wallpaperIntegration->init();
+    // The root of wallpaper packages will be a WallpaperItem we provide the same API via WallpaperIntegration
+    // although with most things nooping
+    constexpr const char *uri = "org.kde.plasma.plasmoid";
+    qmlRegisterType<WallpaperItem>(uri, 2, 0, "WallpaperItem");
+
+    m_wallpaperPackage = KPackage::PackageLoader::self()->loadPackage(QStringLiteral("Plasma/Wallpaper"));
+    m_wallpaperPackage.setPath(KScreenSaverSettingsBase::self()->wallpaperPluginId());
 
     m_lnfIntegration->setPackage(package);
     m_lnfIntegration->setConfig(KScreenSaverSettingsBase::self()->sharedConfig());
@@ -215,8 +221,12 @@ void UnlockApp::initialize()
 
     installEventFilter(this);
 
-    QDBusConnection::sessionBus()
-        .connect(s_plasmaShellService, s_osdServicePath, s_osdServiceInterface, QStringLiteral("osdProgress"), this, SLOT(osdProgress(QString, int, QString)));
+    QDBusConnection::sessionBus().connect(s_plasmaShellService,
+                                          s_osdServicePath,
+                                          s_osdServiceInterface,
+                                          QStringLiteral("osdProgress"),
+                                          this,
+                                          SLOT(osdProgress(QString, int, int, QString)));
     QDBusConnection::sessionBus()
         .connect(s_plasmaShellService, s_osdServicePath, s_osdServiceInterface, QStringLiteral("osdText"), this, SLOT(osdText(QString, QString)));
 
@@ -232,7 +242,7 @@ QWindow *UnlockApp::getActiveScreen()
         return activeScreen;
     }
 
-    for (KQuickAddons::QuickViewSharedEngine *view : qAsConst(m_views)) {
+    for (PlasmaQuick::QuickViewSharedEngine *view : std::as_const(m_views)) {
         if (view->geometry().contains(QCursor::pos())) {
             activeScreen = view;
             break;
@@ -245,33 +255,31 @@ QWindow *UnlockApp::getActiveScreen()
     return activeScreen;
 }
 
-KDeclarative::QmlObjectSharedEngine *UnlockApp::loadWallpaperPlugin(KQuickAddons::QuickViewSharedEngine *view)
+PlasmaQuick::SharedQmlEngine *UnlockApp::loadWallpaperPlugin(PlasmaQuick::QuickViewSharedEngine *view)
 {
-    auto package = m_wallpaperIntegration->package();
-    if (!package.isValid()) {
+    if (!m_wallpaperPackage.isValid()) {
         qCWarning(KSCREENLOCKER_GREET) << "Error loading the wallpaper, no valid package loaded";
         return nullptr;
     }
 
-    auto qmlObject = new KDeclarative::QmlObjectSharedEngine(view);
+    auto qmlObject = new PlasmaQuick::SharedQmlEngine(view);
     qmlObject->setInitializationDelayed(true);
-    qmlObject->setSource(QUrl::fromLocalFile(package.filePath("mainscript")));
-    qmlObject->rootContext()->setContextProperty(QStringLiteral("wallpaper"), m_wallpaperIntegration);
+    qmlObject->setSource(QUrl::fromLocalFile(m_wallpaperPackage.filePath("mainscript")));
     view->setProperty("wallpaperGraphicsObject", QVariant::fromValue(qmlObject));
-    connect(qmlObject, &KDeclarative::QmlObject::finished, this, [this, qmlObject, view] {
-        auto item = qobject_cast<QQuickItem *>(qmlObject->rootObject());
+    connect(qmlObject, &PlasmaQuick::SharedQmlEngine::finished, this, [this, qmlObject, view] {
+        auto item = qobject_cast<WallpaperItem *>(qmlObject->rootObject());
         if (!item) {
-            qCWarning(KSCREENLOCKER_GREET) << "Wallpaper needs to be a QtQuick Item";
+            qCWarning(KSCREENLOCKER_GREET) << "Root item not a WallpaperItem";
             return;
-        }
-
+        };
+        qmlObject->rootContext()->setContextProperty(QStringLiteral("wallpaper"), item);
         view->rootContext()->setContextProperty(QStringLiteral("wallpaper"), item);
-        view->rootContext()->setContextProperty(QStringLiteral("wallpaperIntegration"), m_wallpaperIntegration);
+        view->rootContext()->setContextProperty(QStringLiteral("wallpaperIntegration"), item);
     });
     return qmlObject;
 }
 
-void UnlockApp::setWallpaperItemProperties(KDeclarative::QmlObjectSharedEngine *wallpaperObject, KQuickAddons::QuickViewSharedEngine *view)
+void UnlockApp::setWallpaperItemProperties(PlasmaQuick::SharedQmlEngine *wallpaperObject, PlasmaQuick::QuickViewSharedEngine *view)
 {
     if (!wallpaperObject) {
         return;
@@ -293,6 +301,16 @@ void UnlockApp::setWallpaperItemProperties(KDeclarative::QmlObjectSharedEngine *
 
 void UnlockApp::initialViewSetup()
 {
+    qmlRegisterUncreatableType<PamAuthenticator>("org.kde.kscreenlocker",
+                                                 1,
+                                                 0,
+                                                 "Authenticator",
+                                                 QStringLiteral("authenticators must be obtained from the context"));
+    qmlRegisterUncreatableType<PamAuthenticators>("org.kde.kscreenlocker",
+                                                  1,
+                                                  0,
+                                                  "Authenticators",
+                                                  QStringLiteral("authenticators must be obtained from the context"));
     for (QScreen *screen : screens()) {
         handleScreen(screen);
     }
@@ -315,10 +333,10 @@ void UnlockApp::handleScreen(QScreen *screen)
     });
 }
 
-KQuickAddons::QuickViewSharedEngine *UnlockApp::createViewForScreen(QScreen *screen)
+PlasmaQuick::QuickViewSharedEngine *UnlockApp::createViewForScreen(QScreen *screen)
 {
     // create the view
-    auto *view = new KQuickAddons::QuickViewSharedEngine();
+    auto *view = new PlasmaQuick::QuickViewSharedEngine();
 
     view->setColor(Qt::black);
     view->setScreen(screen);
@@ -328,7 +346,8 @@ KQuickAddons::QuickViewSharedEngine *UnlockApp::createViewForScreen(QScreen *scr
         view->setGeometry(geo);
     });
 
-    view->engine()->rootContext()->setContextObject(new KLocalizedContext(view->engine()));
+    view->engine()->setProperty("_kirigamiTheme", QStringLiteral("KirigamiPlasmaStyle"));
+    view->engine()->rootContext()->setContextObject(new KLocalizedContext(view->engine().get()));
     auto oldFactory = view->engine()->networkAccessManagerFactory();
     view->engine()->setNetworkAccessManagerFactory(nullptr);
     delete oldFactory;
@@ -345,13 +364,14 @@ KQuickAddons::QuickViewSharedEngine *UnlockApp::createViewForScreen(QScreen *scr
     if (m_ksldInterface) {
         view->create();
         org_kde_ksld_x11window(m_ksldInterface, view->winId());
-        wl_display_flush(m_ksldConnection->display());
+        wl_display_flush(m_display);
     }
 
     // engine stuff
     QQmlContext *context = view->engine()->rootContext();
-    connect(view->engine(), &QQmlEngine::quit, this, [this]() {
-        if (m_authenticator->isUnlocked()) {
+    connect(view->engine().get(), &QQmlEngine::quit, this, [this]() {
+        if (m_authenticators->isUnlocked()) {
+            std::cout << "Unlocked" << std::endl;
             QCoreApplication::quit();
         } else {
             qCWarning(KSCREENLOCKER_GREET) << "Greeter tried to quit without being unlocked";
@@ -360,14 +380,14 @@ KQuickAddons::QuickViewSharedEngine *UnlockApp::createViewForScreen(QScreen *scr
 
     context->setContextProperty(QStringLiteral("kscreenlocker_userName"), m_userName);
     context->setContextProperty(QStringLiteral("kscreenlocker_userImage"), m_userImage);
-    context->setContextProperty(QStringLiteral("authenticator"), m_authenticator);
+    context->setContextProperty(QStringLiteral("authenticator"), m_authenticators);
     context->setContextProperty(QStringLiteral("org_kde_plasma_screenlocker_greeter_interfaceVersion"), 2);
     context->setContextProperty(QStringLiteral("org_kde_plasma_screenlocker_greeter_view"), view);
     context->setContextProperty(QStringLiteral("defaultToSwitchUser"), m_defaultToSwitchUser);
     context->setContextProperty(QStringLiteral("config"), m_lnfIntegration->configuration());
 
     auto wallpaperObj = loadWallpaperPlugin(view);
-    if (auto object = view->property("wallpaperGraphicsObject").value<KDeclarative::QmlObjectSharedEngine *>()) {
+    if (auto object = view->property("wallpaperGraphicsObject").value<PlasmaQuick::SharedQmlEngine *>()) {
         // initialize with our size to avoid as much resize events as possible
         object->completeInitialization({
             {QStringLiteral("width"), view->width()},
@@ -381,11 +401,14 @@ KQuickAddons::QuickViewSharedEngine *UnlockApp::createViewForScreen(QScreen *scr
         static const QUrl fallbackUrl(QUrl(QStringLiteral("qrc:/fallbacktheme/LockScreen.qml")));
 
         qCWarning(KSCREENLOCKER_GREET) << "Failed to load lockscreen QML, falling back to built-in locker";
+        for (const auto &error : view->errors()) {
+            qCWarning(KSCREENLOCKER_GREET) << error;
+        }
 
         m_mainQmlPath = fallbackUrl;
         view->setSource(fallbackUrl);
     }
-    view->setResizeMode(KQuickAddons::QuickViewSharedEngine::SizeRootObjectToView);
+    view->setResizeMode(PlasmaQuick::QuickViewSharedEngine::SizeRootObjectToView);
 
     // we need to set this wallpaper properties separately after the lockscreen QML is loaded
     // this is because we need to anchor to the view that gets loaded
@@ -413,16 +436,12 @@ KQuickAddons::QuickViewSharedEngine *UnlockApp::createViewForScreen(QScreen *scr
         if (auto layerShellWindow = LayerShellQt::Window::get(view)) {
             layerShellWindow->setExclusiveZone(-1);
             layerShellWindow->setLayer(LayerShellQt::Window::LayerOverlay);
-            layerShellWindow->setDesiredOutput(screen);
+            layerShellWindow->setKeyboardInteractivity(LayerShellQt::Window::KeyboardInteractivityExclusive);
         }
     }
 
-    // on Wayland we may not use fullscreen as that puts all windows on one screen
-    if (m_testing || QX11Info::isPlatformX11()) {
-        view->show();
-    } else {
-        view->showFullScreen();
-    }
+    // showFullScreen is implicit on X11 (through geometry and hints) and Wayland (layer-shell)
+    view->show();
     view->raise();
 
     auto onFrameSwapped = [this, view] {
@@ -433,7 +452,7 @@ KQuickAddons::QuickViewSharedEngine *UnlockApp::createViewForScreen(QScreen *scr
     return view;
 }
 
-void UnlockApp::markViewsAsVisible(KQuickAddons::QuickViewSharedEngine *view)
+void UnlockApp::markViewsAsVisible(PlasmaQuick::QuickViewSharedEngine *view)
 {
     disconnect(view, &QQuickWindow::frameSwapped, this, nullptr);
     QQmlProperty showProperty(view->rootObject(), QStringLiteral("viewVisible"));
@@ -465,7 +484,7 @@ void UnlockApp::getFocus()
     }
     // this loop is required to make the qml/graphicsscene properly handle the shared keyboard input
     // ie. "type something into the box of every greeter"
-    for (KQuickAddons::QuickViewSharedEngine *view : qAsConst(m_views)) {
+    for (PlasmaQuick::QuickViewSharedEngine *view : std::as_const(m_views)) {
         if (!m_testing) {
             view->setKeyboardGrabEnabled(true); // TODO - check whether this still works in master!
         }
@@ -483,7 +502,7 @@ void UnlockApp::setLockedPropertyOnViews()
     delete m_delayedLockTimer;
     m_delayedLockTimer = nullptr;
 
-    for (KQuickAddons::QuickViewSharedEngine *view : qAsConst(m_views)) {
+    for (PlasmaQuick::QuickViewSharedEngine *view : std::as_const(m_views)) {
         QQmlProperty lockProperty(view->rootObject(), QStringLiteral("locked"));
         lockProperty.write(true);
     }
@@ -502,6 +521,7 @@ void UnlockApp::suspendToRam()
 
     m_ignoreRequests = true;
     m_resetRequestIgnoreTimer->start();
+    m_authenticators->cancel();
 
     PowerManagement::instance()->suspend();
 }
@@ -514,6 +534,7 @@ void UnlockApp::suspendToDisk()
 
     m_ignoreRequests = true;
     m_resetRequestIgnoreTimer->start();
+    m_authenticators->cancel();
 
     PowerManagement::instance()->hibernate();
 }
@@ -526,11 +547,11 @@ void UnlockApp::setTesting(bool enable)
     }
     if (enable) {
         // remove bypass window manager hint
-        for (KQuickAddons::QuickViewSharedEngine *view : qAsConst(m_views)) {
+        for (PlasmaQuick::QuickViewSharedEngine *view : std::as_const(m_views)) {
             view->setFlags(view->flags() & ~Qt::X11BypassWindowManagerHint);
         }
     } else {
-        for (KQuickAddons::QuickViewSharedEngine *view : qAsConst(m_views)) {
+        for (PlasmaQuick::QuickViewSharedEngine *view : std::as_const(m_views)) {
             view->setFlags(view->flags() | Qt::X11BypassWindowManagerHint);
         }
     }
@@ -563,8 +584,8 @@ void UnlockApp::lockImmediately()
 bool UnlockApp::eventFilter(QObject *obj, QEvent *event)
 {
     if (obj != this && event->type() == QEvent::Show) {
-        KQuickAddons::QuickViewSharedEngine *view = nullptr;
-        for (KQuickAddons::QuickViewSharedEngine *v : qAsConst(m_views)) {
+        PlasmaQuick::QuickViewSharedEngine *view = nullptr;
+        for (PlasmaQuick::QuickViewSharedEngine *v : std::as_const(m_views)) {
             if (v == obj) {
                 view = v;
                 break;
@@ -587,12 +608,12 @@ bool UnlockApp::eventFilter(QObject *obj, QEvent *event)
     }
 
     if (event->type() == QEvent::KeyPress) { // react if saver is visible
-        shareEvent(event, qobject_cast<KQuickAddons::QuickViewSharedEngine *>(obj));
+        shareEvent(event, qobject_cast<PlasmaQuick::QuickViewSharedEngine *>(obj));
         return false; // we don't care
     } else if (event->type() == QEvent::KeyRelease) { // conditionally reshow the saver
         QKeyEvent *ke = static_cast<QKeyEvent *>(event);
         if (ke->key() != Qt::Key_Escape) {
-            shareEvent(event, qobject_cast<KQuickAddons::QuickViewSharedEngine *>(obj));
+            shareEvent(event, qobject_cast<PlasmaQuick::QuickViewSharedEngine *>(obj));
             return false; // irrelevant
         } else {
             auto dpms = new KScreen::Dpms(this);
@@ -615,7 +636,7 @@ bool UnlockApp::eventFilter(QObject *obj, QEvent *event)
  * even if the focus is actually on a powered off screen.
  */
 
-void UnlockApp::shareEvent(QEvent *e, KQuickAddons::QuickViewSharedEngine *from)
+void UnlockApp::shareEvent(QEvent *e, PlasmaQuick::QuickViewSharedEngine *from)
 {
     // from can be NULL any time (because the parameter is passed as qobject_cast)
     // m_views.contains(from) is atm. supposed to be true but required if any further
@@ -626,7 +647,7 @@ void UnlockApp::shareEvent(QEvent *e, KQuickAddons::QuickViewSharedEngine *from)
         // Any change in regarded event processing shall be tested thoroughly!
         removeEventFilter(this); // prevent recursion!
         const bool accepted = e->isAccepted(); // store state
-        for (KQuickAddons::QuickViewSharedEngine *view : qAsConst(m_views)) {
+        for (PlasmaQuick::QuickViewSharedEngine *view : std::as_const(m_views)) {
             if (view != from) {
                 QCoreApplication::sendEvent(view, e);
                 e->setAccepted(accepted);
@@ -660,59 +681,40 @@ void UnlockApp::setDefaultToSwitchUser(bool defaultToSwitchUser)
 
 void UnlockApp::setKsldSocket(int socket)
 {
-    using namespace KWayland::Client;
-    m_ksldConnection = new ConnectionThread;
-    m_ksldConnection->setSocketFd(socket);
-
-    m_ksldRegistry = new Registry();
-    EventQueue *queue = new EventQueue(m_ksldRegistry);
-
-    connect(m_ksldRegistry, &Registry::interfaceAnnounced, this, [this, queue](QByteArray interface, quint32 name, quint32 version) {
+    m_display = wl_display_connect_to_fd(socket);
+    auto socketnotifier = new QSocketNotifier(socket, QSocketNotifier::Read, this);
+    connect(socketnotifier, &QSocketNotifier::activated, this, [this] {
+        wl_display_dispatch(m_display);
+    });
+    auto registry = wl_display_get_registry(m_display);
+    auto globalAdded = [](void *data, struct wl_registry *registry, uint32_t name, const char *interface, uint32_t version) {
         Q_UNUSED(version)
-        if (interface != QByteArrayLiteral("org_kde_ksld")) {
+        if (interface != "org_kde_ksld"_ba) {
             return;
         }
+        auto self = static_cast<UnlockApp *>(data);
         // bind version 1 as we dropped all the V2 features
-        m_ksldInterface = reinterpret_cast<org_kde_ksld *>(wl_registry_bind(*m_ksldRegistry, name, &org_kde_ksld_interface, 1));
-        queue->addProxy(m_ksldInterface);
-
-        for (auto v : qAsConst(m_views)) {
-            org_kde_ksld_x11window(m_ksldInterface, v->winId());
-            wl_display_flush(m_ksldConnection->display());
+        self->m_ksldInterface = static_cast<org_kde_ksld *>(wl_registry_bind(registry, name, &org_kde_ksld_interface, 1));
+        for (auto v : std::as_const(self->m_views)) {
+            org_kde_ksld_x11window(self->m_ksldInterface, v->winId());
+            wl_display_flush(self->m_display);
         }
-    });
-
-    connect(
-        m_ksldConnection,
-        &ConnectionThread::connected,
-        this,
-        [this, queue] {
-            m_ksldRegistry->create(m_ksldConnection);
-            queue->setup(m_ksldConnection);
-            m_ksldRegistry->setEventQueue(queue);
-            m_ksldRegistry->setup();
-            wl_display_flush(m_ksldConnection->display());
-        },
-        Qt::QueuedConnection);
-
-    m_ksldConnectionThread = new QThread(this);
-    m_ksldConnection->moveToThread(m_ksldConnectionThread);
-    m_ksldConnectionThread->start();
-    m_ksldConnection->initConnection();
+    };
+    auto noopGlobalRemove = [](void *, struct wl_registry *, uint32_t) {};
+    static const wl_registry_listener registryListener = wl_registry_listener{globalAdded, noopGlobalRemove};
+    wl_registry_add_listener(registry, &registryListener, this);
+    wl_display_flush(m_display);
 }
 
-void UnlockApp::osdProgress(const QString &icon, int percent, const QString &additionalText)
+void UnlockApp::osdProgress(const QString &icon, int percent, int maximumPercent, const QString &additionalText)
 {
-    for (auto v : qAsConst(m_views)) {
+    for (auto v : std::as_const(m_views)) {
         auto osd = v->rootObject()->findChild<QQuickItem *>(QStringLiteral("onScreenDisplay"));
         if (!osd) {
             continue;
         }
         osd->setProperty("osdValue", percent);
-        // HACK: if the value is >100 assume max is 150, to fix https://bugs.kde.org/show_bug.cgi?id=430536
-        // this is because the osdProgress signal does not have a maxPercent parameter
-        // it can't be fixed without breaking the DBus API so TODO KF6
-        osd->setProperty("osdMaxValue", percent > 100 ? 150 : 100);
+        osd->setProperty("osdMaxValue", maximumPercent);
         osd->setProperty("osdAdditionalText", additionalText);
         osd->setProperty("showingProgress", true);
         osd->setProperty("icon", icon);
@@ -722,7 +724,7 @@ void UnlockApp::osdProgress(const QString &icon, int percent, const QString &add
 
 void UnlockApp::osdText(const QString &icon, const QString &additionalText)
 {
-    for (auto v : qAsConst(m_views)) {
+    for (auto v : std::as_const(m_views)) {
         auto osd = v->rootObject()->findChild<QQuickItem *>(QStringLiteral("onScreenDisplay"));
         if (!osd) {
             continue;
@@ -751,3 +753,5 @@ void UnlockApp::updateCanHibernate()
 }
 
 } // namespace
+
+#include "greeterapp.moc"
