@@ -5,8 +5,8 @@ SPDX-License-Identifier: GPL-2.0-or-later
 */
 #pragma once
 
-#include <KDeclarative/QmlObjectSharedEngine>
 #include <KPackage/PackageStructure>
+#include <PlasmaQuick/SharedQmlEngine>
 #include <QGuiApplication>
 #include <QUrl>
 
@@ -19,7 +19,7 @@ class Registry;
 }
 }
 
-namespace KQuickAddons
+namespace PlasmaQuick
 {
 class QuickViewSharedEngine;
 }
@@ -28,7 +28,7 @@ class Authenticator;
 
 struct org_kde_ksld;
 
-class PamAuthenticator;
+class PamAuthenticators;
 
 namespace ScreenLocker
 {
@@ -57,7 +57,7 @@ public:
     void updateCanHibernate();
 
 public Q_SLOTS:
-    void osdProgress(const QString &icon, int percent, const QString &additionalText);
+    void osdProgress(const QString &icon, int percent, const int maximumPercent, const QString &additionalText);
     void osdText(const QString &icon, const QString &additionalText);
 
 protected:
@@ -65,25 +65,25 @@ protected:
 
 private Q_SLOTS:
     void handleScreen(QScreen *screen);
-    KQuickAddons::QuickViewSharedEngine *createViewForScreen(QScreen *screen);
+    PlasmaQuick::QuickViewSharedEngine *createViewForScreen(QScreen *screen);
     void resetRequestIgnore();
     void suspendToRam();
     void suspendToDisk();
     void getFocus();
-    void markViewsAsVisible(KQuickAddons::QuickViewSharedEngine *view);
+    void markViewsAsVisible(PlasmaQuick::QuickViewSharedEngine *view);
     void setLockedPropertyOnViews();
 
 private:
     void initialize();
-    void shareEvent(QEvent *e, KQuickAddons::QuickViewSharedEngine *from);
-    KDeclarative::QmlObjectSharedEngine *loadWallpaperPlugin(KQuickAddons::QuickViewSharedEngine *view);
-    void setWallpaperItemProperties(KDeclarative::QmlObjectSharedEngine *wallpaperObject, KQuickAddons::QuickViewSharedEngine *view);
+    void shareEvent(QEvent *e, PlasmaQuick::QuickViewSharedEngine *from);
+    PlasmaQuick::SharedQmlEngine *loadWallpaperPlugin(PlasmaQuick::QuickViewSharedEngine *view);
+    void setWallpaperItemProperties(PlasmaQuick::SharedQmlEngine *wallpaperObject, PlasmaQuick::QuickViewSharedEngine *view);
     void screenGeometryChanged(QScreen *screen, const QRect &geo);
     QWindow *getActiveScreen();
 
     QString m_packageName;
     QUrl m_mainQmlPath;
-    QList<KQuickAddons::QuickViewSharedEngine *> m_views;
+    QList<PlasmaQuick::QuickViewSharedEngine *> m_views;
     QTimer *m_resetRequestIgnoreTimer;
     QTimer *m_delayedLockTimer;
     KPackage::Package m_package;
@@ -91,7 +91,7 @@ private:
     bool m_ignoreRequests;
     bool m_immediateLock;
     bool m_runtimeInitialized;
-    PamAuthenticator *m_authenticator;
+    PamAuthenticators *m_authenticators;
     int m_graceTime;
     bool m_noLock;
     bool m_defaultToSwitchUser;
@@ -100,12 +100,10 @@ private:
     bool m_canHibernate = false;
     QString m_userName, m_userImage;
 
-    KWayland::Client::ConnectionThread *m_ksldConnection = nullptr;
-    KWayland::Client::Registry *m_ksldRegistry = nullptr;
-    QThread *m_ksldConnectionThread = nullptr;
+    wl_display *m_display = nullptr;
     org_kde_ksld *m_ksldInterface = nullptr;
 
-    WallpaperIntegration *m_wallpaperIntegration;
+    KPackage::Package m_wallpaperPackage;
     LnFIntegration *m_lnfIntegration;
 };
 } // namespace
