@@ -70,7 +70,7 @@ int main(int argc, char *argv[])
 
     qCDebug(KSCREENLOCKER_GREET) << "Greeter is starting up.";
 
-    KLocalizedString::setApplicationDomain("kscreenlocker_greet");
+    KLocalizedString::setApplicationDomain(QByteArrayLiteral("kscreenlocker_greet"));
 
     // explicitly disable input methods on x11 as it makes it impossible to unlock, see BUG 306932
     // but explicitly set on screen keyboard such as maliit is allowed
@@ -82,6 +82,8 @@ int main(int argc, char *argv[])
 
     // Suppresses modal warnings about unwritable configuration files which may render the system inaccessible
     qputenv("KDE_HOME_READONLY", "1");
+    // Kwin will re-lock if it restarts, reconnecting would leave us with two greeters but only one functional
+    qunsetenv("QT_WAYLAND_RECONNECT");
     // Disable QML caching to prevent cache corruption in full or near-full disk scenarios.
     // https://bugs.kde.org/show_bug.cgi?id=471952
     // https://bugreports.qt.io/browse/QTBUG-117130
@@ -100,6 +102,7 @@ int main(int argc, char *argv[])
     QObject::connect(KSignalHandler::self(), &KSignalHandler::signalReceived, &app, &signalHandler);
 
     app.setQuitOnLastWindowClosed(false);
+    app.setQuitLockEnabled(false);
     QCoreApplication::setApplicationName(QStringLiteral("kscreenlocker_greet"));
     QCoreApplication::setApplicationVersion(QStringLiteral("0.1"));
     QCoreApplication::setOrganizationDomain(QStringLiteral("kde.org"));
