@@ -127,7 +127,7 @@ void AppearanceSettings::loadWallpaperConfig()
         delete m_wallpaperIntegration;
     }
 
-    m_wallpaperIntegration = new ScreenLocker::WallpaperIntegration(this);
+    m_wallpaperIntegration = new ScreenLocker::WallpaperIntegration();
     m_wallpaperIntegration->setConfig(KScreenSaverSettings::getInstance().sharedConfig());
     m_wallpaperIntegration->setPluginName(KScreenSaverSettings::getInstance().wallpaperPluginId());
     m_wallpaperIntegration->init();
@@ -160,3 +160,5 @@ void AppearanceSettings::loadLnfConfig()
     auto sourceFile = m_package.fileUrl(QByteArrayLiteral("lockscreen"), QStringLiteral("config.qml"));
     m_lnfConfigFile = sourceFile;
 }
+
+#include "moc_appearancesettings.cpp"
